local gr = love.graphics
require 'luafft'
local new = complex.new

function love.load()
	love.mouse.setVisible( false )
	gr.setColor( 0, 255, 0 )
	Initialize()
	love.window.setTitle( 'SpectrumSurge by Michael Lutz and David Manura' )
	files 		= love.filesystem.getDirectoryItems( 'music' )
	sel 		= 1
	playMusic( sel, files )
	line		= true
	fr			= 8
	end

function love.update() spectro_up( sound, soundData ) end

function love.draw()
	gr.setColor( 55, 155, 155, 256 )
	gr.rectangle( 'line', win.x, win.y, win.w, win.h )
	gr.print( files[sel], win.x, win.h + win.y + 20 )
	freqBars()
	gr.setColor(0,255,0)
	spectro_show()
	end

function Initialize()
	if fullscreen then love.window.setFullscreen( true ) else love.window.setMode(800,600) end
	screen		= { w = gr.getWidth(), h = gr.getHeight() }
	Size		= 1024
	scale		= { x = math.ceil(6*screen.w/Size), y = 20 }
	win			= { x=(screen.w-scale.x*Size/8)/2, y=screen.h/10, w=scale.x*Size/8, h=screen.h-screen.h/5 }
	end

function freqBars()
	local st = 4000/176.4	-- magic numbers
	for i=1,5 do x = scale.x * i * st + win.x gr.line( x, win.y, x, win.y + win.h ) end
	end

function love.keyreleased( key )
	if key == 'escape' then love.event.quit() end
	if key == 'l' then line = not line end
	if key == 'f11' then fullscreen = not fullscreen Initialize() end
	end

function playMusic( n, tab )
	soundData	= love.sound.newSoundData( 'music/'..tab[n] )
	sound		= love.audio.newSource( soundData )
	sound:play()
	end

function spectro_up( obj, sdata )
	local MusicPos	= obj:tell( 'samples' )
	local MusicSize	= sdata:getSampleCount()
	if MusicPos >= MusicSize - Size then
		scale.y = 20
		sel = sel < #files and sel+1 or 1
		playMusic( sel, files )
		--love.audio.rewind(sound)
		end
	local List = {}
	for i= MusicPos, MusicPos + (Size-1) do
		if i + Size > MusicSize then i = MusicSize-Size end
		if sdata:getChannels()==1 then
			List[#List+1] = new(sdata:getSample(i), 0)
			else
			List[#List+1] = new(sdata:getSample(i*2), 0)
			end
		end
	spectrum = fft( List, false )
	end

function spectro_show()
	for i = 1, #spectrum/fr do
		local freq  = spectrum[i]:abs()
		local freq2 = spectrum[i+1]:abs()
		if scale.y*freq > win.h/2 then scale.y = (win.h/2)/freq end
		if line then
			gr.rectangle('fill',win.w*math.log(i)/6.3+win.x, screen.h-2*scale.y*freq-win.y,  2, 2*scale.y*freq)
			fr=2
			gr.line( win.w*math.log(i)/6.3+win.x, screen.h-2*scale.y*freq-win.y,win.w*math.log(i+1)/6.3+win.x,screen.h-2*scale.y*freq2-win.y )
			else
			fr=8
			gr.ellipse( 'line', i* scale.x+win.x, screen.h/2,  scale.x/6, -scale.y*freq )
			--gr.ellipse( 'line', 100*math.log(i)+win.x, screen.h/2,  2, -scale.y*freq )
			end
		end
	end
